@@ -28,6 +28,9 @@ public:
 public slots:
     void startIt();
 
+protected:
+    void closeEvent(QCloseEvent *event);
+
 private:
     Ui::MainWindow *ui;
     Ui::Prefs      *prefs;
