@@ -1,12 +1,15 @@
+
 #include "Morse.h"
 
 #include <QtCore/QSettings>
 #include <QtGui/QMenu>
 #include <QtGui/QMenuBar>
+#include <QtGui/QSizePolicy>
 #include <qdebug.h>
 
 #include "MainWindow.h"
 
+#include "modes/FrontPage.h"
 #include "modes/PlayMode.h"
 #include "modes/LetterTrainingMode.h"
 #include "modes/SpeedTrainingMode.h"
@@ -23,7 +26,7 @@ Morse::Morse()
     : QObject(), m_sequenceLabel(0), m_parent(0), m_audioOutput(),
       m_dahMult(3), m_pauseMult(1), m_letterPauseMult(3), m_spaceMult(7),
       m_dit(0), m_dah(0), m_space(0), m_pause(0), m_letterPause(0), m_playBuffer(0), m_playingMode(STOPPED), m_gameMode(PLAY),
-    m_currentWPMGoal(WPMGOAL), m_currentWPMAccept(WPMACCEPT), m_ui(0), m_tone(DEFAULT_TONE)
+    m_currentWPMGoal(WPMGOAL), m_currentWPMAccept(WPMACCEPT), m_ui(0), m_tone(DEFAULT_TONE), m_leadInPause(0,0), m_signalMapper(new QSignalMapper(this))
 {
     qDebug() << "new morse";
     m_modes.insert(PLAY, new PlayMode(this, m_ui));
@@ -35,7 +38,7 @@ Morse::Morse(MainWindow *parent, QAudioOutput *output, Ui::MainWindow *ui)
       m_dahMult(3), m_pauseMult(1), m_letterPauseMult(3), m_spaceMult(7),
       m_dit(0), m_dah(0), m_space(0), m_pause(0), m_letterPause(0), m_playBuffer(0), m_playingMode(STOPPED), m_gameMode(PLAY),
       m_currentWPMGoal(WPMGOAL), m_currentWPMAccept(WPMACCEPT),
-      m_ui(ui), m_tone(DEFAULT_TONE)
+      m_ui(ui), m_tone(DEFAULT_TONE), m_leadInPause(0,0), m_signalMapper(new QSignalMapper(this))
 {
 
     qDebug() << "new morse2";
@@ -44,8 +47,11 @@ Morse::Morse(MainWindow *parent, QAudioOutput *output, Ui::MainWindow *ui)
 
     createTones(WPMGOAL);
     qsrand(QTime::currentTime().msec());
-    loadSettings();
 
+    qDebug() << "original buffer size: " << m_audioOutput->bufferSize();
+    m_audioOutput->setBufferSize(qMin(m_audioOutput->bufferSize() * 4, 1024*64));
+
+    m_modes.insert(FRONTPAGE, new FrontPage(this, m_ui));
     m_modes.insert(PLAY, new PlayMode(this, m_ui));
     m_modes.insert(TRAIN, new LetterTrainingMode(this, m_ui));
     m_modes.insert(SPEEDTRAIN, new SpeedTrainingMode(this, m_ui));
@@ -55,10 +61,13 @@ Morse::Morse(MainWindow *parent, QAudioOutput *output, Ui::MainWindow *ui)
     m_modes.insert(GROUPGAME, new GroupGame(this, m_ui));
     m_modes.insert(READ, new ReadMode(this, m_ui));
 
-    switchMode(Morse::PLAY);
+    loadSettings();
 
-    connect(m_ui->clearTraining, SIGNAL(clicked()), this, SLOT(clearStatsButton()));
-    connect(m_ui->play, SIGNAL(clicked()), this, SLOT(playButton()));
+    switchMode(Morse::FRONTPAGE);
+}
+
+MorseMode *Morse::getMode(TrainingMode which) const {
+    return m_modes[which];
 }
 
 void Morse::prefsButton() {
@@ -69,14 +78,14 @@ void Morse::prefsButton() {
     prefsDialog.WPMGoal->setValue(m_currentWPMGoal);
     prefsDialog.tone->setValue(m_tone);
 
-    prefsDialog.weighting->insertItem(LOW, "Low");
-    prefsDialog.weighting->insertItem(HIGH, "High");
-    prefsDialog.weighting->setCurrentIndex(m_badLetterWeighting);
+    //prefsDialog.weighting->insertItem(LOW, "Low");
+    //prefsDialog.weighting->insertItem(HIGH, "High");
+    //prefsDialog.weighting->setCurrentIndex(m_badLetterWeighting);
 
     if (dialog->exec() == QDialog::Accepted) {
         m_currentWPMAccept = prefsDialog.WPMAccepted->value();
         m_currentWPMGoal = prefsDialog.WPMGoal->value();
-        m_badLetterWeighting = (BadLetterWeighting) prefsDialog.weighting->currentIndex();
+        //m_badLetterWeighting = (BadLetterWeighting) prefsDialog.weighting->currentIndex();
         m_tone = prefsDialog.tone->value();
         saveSettings();
         loadSettings();
@@ -88,6 +97,9 @@ void Morse::aboutButton() {
     QDialog *dialog = new QDialog(m_parent);
     about.setupUi(dialog);
     about.helpicon->setPixmap(QPixmap(":/icons/64x64/cutecw.png"));
+#ifdef SMALL_DEVICE
+    dialog->setAttribute(Qt::WA_Maemo5StackedWindow);
+#endif
     dialog->exec();
 }
 
@@ -102,16 +114,24 @@ void Morse::saveSettings() {
     settings.setValue("WPM/Goal", m_currentWPMGoal);
     settings.setValue("WPM/Accept", m_currentWPMAccept);
     settings.setValue("Tone", m_tone);
-    settings.setValue("LetterWeighting", int(m_badLetterWeighting));
-    qDebug() << "saving: " << m_badLetterWeighting;
+    //settings.setValue("LetterWeighting", int(m_badLetterWeighting));
+
+    for(int i = PLAY; i <= maximumTrainingMode; i++) {
+        m_modes[(TrainingMode) i]->saveSettings(settings);
+    }
 }
 
 void Morse::loadSettings() {
     QSettings settings("WS6Z", "qtcw");
     m_currentWPMGoal = settings.value("WPM/Goal", WPMGOAL).toInt();
     m_currentWPMAccept = settings.value("WPM/Accept", WPMACCEPT).toInt();
-    m_badLetterWeighting = (BadLetterWeighting) settings.value("LetterWeighting", HIGH).toInt();
+    //m_badLetterWeighting = (BadLetterWeighting) settings.value("LetterWeighting", HIGH).toInt();
     m_tone = settings.value("Tone", DEFAULT_TONE).toInt();
+    for(int i = PLAY; i <= maximumTrainingMode; i++) {
+        if (! m_modes.contains((TrainingMode) i))
+            continue;
+        m_modes[(TrainingMode) i]->loadSettings(settings);
+    }
     createTones(m_currentWPMGoal);  
 }
 
@@ -137,9 +157,9 @@ QTime Morse::maybePlaySequence(bool addPause) {
     if (m_playingMode == STOPPED || m_playingMode == PAUSED) {
         m_playBuffer->restartData();
         QTime playTime = sequenceTime();
-        playSequence();
         if (addPause)
-            add(pause());
+            add(letterPause());
+        playSequence();
         return QTime::currentTime().addSecs(playTime.second()).addMSecs(playTime.msec());
     }
     return QTime(0,0,0);
@@ -206,9 +226,8 @@ void Morse::switchMode(int newmode) {
 
     m_modes[m_gameMode]->switchFromYou();
     m_gameMode = (Morse::TrainingMode) newmode;
+    setupWidgets();
     m_modes[(TrainingMode) newmode]->switchToYou();
-
-    connect(menuBar()->addAction("About"), SIGNAL(triggered()), this, SLOT(aboutButton()));
 }
 
 //
@@ -227,8 +246,9 @@ QTime Morse::addAndPlayIt(QChar c) {
 
 QTime Morse::playIt(QChar c) {
     clearList();
-    add(pause());  // allows audio device to kick in (otherwise distortion can occur)
-    add(c, false);
+    add(letterPause());  // allows audio device to kick in (otherwise distortion can occur)
+    m_leadInPause = sequenceTime();
+    add(c, false); // add the pause in maybePlaySequence so it can calculate the sequence end
     return maybePlaySequence(true);
 }
 
@@ -412,3 +432,152 @@ Morse::ditSecs()
 {
     return m_ditSecs;
 }
+
+void Morse::setupWidgets()
+{
+    QWidget *theMainThing = m_ui->centralWidget = new QWidget(m_parent);
+    QVBoxLayout *topvbox = m_ui->verticalLayout = new QVBoxLayout(theMainThing);
+
+    QHBoxLayout *buttonHBox = m_ui->horizontalLayout = new QHBoxLayout(theMainThing);
+    topvbox->addLayout(buttonHBox);
+
+    setupTopButtons(buttonHBox);
+    topvbox->addWidget(m_ui->helpBar = new QLabel("help"));
+    setupWPMLayout(topvbox, theMainThing);
+    topvbox->addLayout(m_ui->forModes = new QHBoxLayout(theMainThing));
+    topvbox->setStretchFactor(m_ui->forModes, 10);
+    setupSequenceLayouts(topvbox, theMainThing);
+
+    setupConnections();
+
+    m_parent->setCentralWidget(theMainThing);
+}
+
+
+void Morse::setupWPMLayout(QVBoxLayout *parentLayout, QWidget *theMainThing)
+{
+    QHBoxLayout *WPMLayout = m_ui->horizontalLayout_5 = new QHBoxLayout(theMainThing);
+    parentLayout->addLayout(WPMLayout);
+
+    WPMLayout->addWidget(m_ui->WPM = new QLabel(""));
+    WPMLayout->addWidget(m_ui->letter = new QLabel(""));
+
+    QFont font = m_ui->WPM->font();
+    font.setPointSize(font.pointSize() * 2); // A cheap way of getting bigger regardless of the default stylee
+    font.setBold(true);
+    m_ui->WPM->setFont(font);
+
+    font = m_ui->letter->font();
+    font.setPointSize(font.pointSize() * 2);
+    font.setBold(true);
+    m_ui->letter->setFont(font);
+}
+
+void Morse::setupSequenceLayouts(QVBoxLayout *parentLayout, QWidget *theMainThing)
+{
+    QHBoxLayout *sequenceLayout = m_ui->horizontalLayout_3 = new QHBoxLayout(theMainThing);
+    parentLayout->addLayout(sequenceLayout);
+    sequenceLayout->addWidget(m_ui->label = new QLabel(tr("Sequence:")));
+    sequenceLayout->addWidget(m_sequenceLabel = m_ui->sequence = new QLabel(tr("")));
+
+    QHBoxLayout *lastWPMLayout = m_ui->horizontalLayout_4 = new QHBoxLayout(theMainThing);
+    parentLayout->addLayout(lastWPMLayout);
+    lastWPMLayout->addWidget(m_ui->label_3 = new QLabel(tr("Last WPM:")));
+    lastWPMLayout->addWidget(m_ui->lastwpm = new QLabel(tr("")));
+
+    QHBoxLayout *aveWPMLayout = m_ui->horizontalLayout_5 = new QHBoxLayout(theMainThing);
+    parentLayout->addLayout(aveWPMLayout);
+    aveWPMLayout->addWidget(m_ui->label_3 = new QLabel(tr("Average WPM:")));
+    aveWPMLayout->addWidget(m_ui->avewpm = new QLabel(tr("")));
+}
+
+void Morse::setupTopButtons(QLayout *parentLayout)
+{
+    QPushButton *button;
+    QSizePolicy policy;
+    QIcon homeIcon(":/icons/go-home.png");
+
+    button = m_ui->homeButton = new QPushButton(homeIcon, tr("Home"));
+    policy = button->sizePolicy();
+    policy.setHorizontalPolicy(QSizePolicy::Fixed);
+    button->setSizePolicy(policy);
+    parentLayout->addWidget(button);
+    connect(button, SIGNAL(clicked()), this, SLOT(goHome()));
+
+    button = m_ui->play = new QPushButton(tr("Play"));
+    parentLayout->addWidget(button);
+    m_ui->play->setIcon(QIcon(":/icons/play.png"));
+
+    button = m_ui->changeSequence = new QPushButton(tr("Sequence"));
+    parentLayout->addWidget(button);
+
+    button = m_ui->changeWords = new QPushButton(tr("Change Words"));
+    parentLayout->addWidget(button);
+}
+
+void Morse::createModesMenuButton(QPushButton *modeButton) {
+    // Create the "mode" menu
+    QMenu *modeMenu = new QMenu(modeButton);
+    modeButton->setMenu(modeMenu);
+
+    createModesMenu(modeMenu);
+}
+
+void Morse::createModesMenu(QMenu *modeMenu) {
+
+    QAction *action = modeMenu->addAction(m_modes[PLAY]->name());
+    connect(action, SIGNAL(triggered()), m_signalMapper, SLOT(map()));
+    m_signalMapper->setMapping(action, (int) Morse::PLAY);
+
+    QMenu *trainingMenu = modeMenu->addMenu(tr("Training"));
+
+    action = trainingMenu->addAction(m_modes[TRAIN]->name());
+    connect(action, SIGNAL(triggered()), m_signalMapper, SLOT(map()));
+    m_signalMapper->setMapping(action, (int) Morse::TRAIN);
+
+    action = trainingMenu->addAction(m_modes[SPEEDTRAIN]->name());
+    connect(action, SIGNAL(triggered()), m_signalMapper, SLOT(map()));
+    m_signalMapper->setMapping(action, (int) Morse::SPEEDTRAIN);
+
+    action = trainingMenu->addAction(m_modes[WORDS]->name());
+    connect(action, SIGNAL(triggered()), m_signalMapper, SLOT(map()));
+    m_signalMapper->setMapping(action, (int) Morse::WORDS);
+
+    action = trainingMenu->addAction(m_modes[GROUPS]->name());
+    connect(action, SIGNAL(triggered()), m_signalMapper, SLOT(map()));
+    m_signalMapper->setMapping(action, (int) Morse::GROUPS);
+
+    QMenu *gamesMenu = modeMenu->addMenu(tr("Games"));
+
+    action = gamesMenu->addAction(m_modes[WORDGAME]->name());
+    connect(action, SIGNAL(triggered()), m_signalMapper, SLOT(map()));
+    m_signalMapper->setMapping(action, (int) Morse::WORDGAME);
+
+    action = gamesMenu->addAction(m_modes[GROUPGAME]->name());
+    connect(action, SIGNAL(triggered()), m_signalMapper, SLOT(map()));
+    m_signalMapper->setMapping(action, (int) Morse::GROUPGAME);
+
+    action = modeMenu->addAction(m_modes[READ]->name());
+    connect(action, SIGNAL(triggered()), m_signalMapper, SLOT(map()));
+    m_signalMapper->setMapping(action, (int) Morse::READ);
+
+    connect(m_signalMapper, SIGNAL(mapped(int)), this, SLOT(switchMode(int)));
+#ifdef SMALL_DEVICE
+    m_ui->modeMenu->setText(m_modes[PLAY]->name());
+#endif
+}
+
+void Morse::setupConnections()
+{
+    connect(m_ui->play, SIGNAL(clicked()), this, SLOT(playButton()));
+}
+
+MainWindow *Morse::parent()
+{
+    return m_parent;
+}
+
+void Morse::goHome()
+{
+    switchMode(FRONTPAGE);
+}
