@@ -38,10 +38,20 @@ public:
 
     enum ditdah{ DIT, DAH, SPACE, PAUSE };
 
-    enum TrainingMode { PLAY, TRAIN, SPEEDTRAIN, WORDS, GROUPS, WORDGAME, GROUPGAME, READ, TEST };
+    enum TrainingMode { FRONTPAGE, PLAY, TRAIN, SPEEDTRAIN, WORDS, GROUPS, WORDGAME, GROUPGAME, READ }; // eventually: TEST
+    static const int maximumTrainingMode = READ;
     enum AudioMode { STOPPED, PLAYING, PAUSED };
     enum BadLetterWeighting { LOW = 1, HIGH = 2 };
 
+    void setupWidgets();
+    void setupTopButtons(QLayout *parentLayout);
+    void createModesMenuButton(QPushButton *modeButton);
+    void createModesMenu(QMenu *modeButton);
+    void setupSequenceLayouts(QVBoxLayout *parentLayout, QWidget *theMainThing);
+    void setupWPMLayout(QVBoxLayout *parentLayout, QWidget *theMainThing);
+    void setupConnections();
+    MorseMode * getMode(TrainingMode) const;
+
     void setupWords();
     void setupSequences();
     void clearList();
@@ -77,6 +87,8 @@ public:
     TrainingMode trainingMode();
     BadLetterWeighting badLetterWeighting();
 
+    MainWindow *parent();
+
 public slots:
 
     void playSequence();
@@ -86,6 +98,7 @@ public slots:
     void audioFinished(QAudio::State state);
     void keyPressed(QString newtext);
     void keyPressed(QChar key);
+    void goHome();
 
     void prefsButton();
     void aboutButton();
@@ -126,6 +139,9 @@ private:
     BadLetterWeighting               m_badLetterWeighting;
     QTimer                           m_timer;
     int                              m_tone;
+    QTime                            m_leadInPause;
+
+    QSignalMapper                   *m_signalMapper;
 };
 
 #endif // MORSE_H
