@@ -16,13 +16,20 @@ public:
     virtual void play();
     virtual void handleKeyPress(QChar letterPressed);
 
+    virtual QString name();
     virtual QString helpText();
 
+    virtual QTime startNextTrainingKey();
     void startTimerToNextKey(QTime plusTime);
+    virtual bool elapsedTimeWasTooLong(int msElapsed, MorseStat *stat);
+
+    virtual void loadSettings(QSettings &settings);
+    virtual void saveSettings(QSettings &settings);
 
 public slots:
     void startTimerToNextKey(int plusMSecs = 0);
     void startNextSpeedKey();
+    virtual void updateGraphs();
 };
 
 #endif // SPEEDTRAININGMODE_H
