@@ -24,8 +24,12 @@ public:
 
     void handleWordResponse(QChar letter);
 
+    virtual QString name();
     virtual QString helpText();
 
+    virtual void loadSettings(QSettings &settings);
+    virtual void saveSettings(QSettings &settings);
+
 signals:
    void groupEntered(int right, int total);
 
