@@ -14,6 +14,7 @@ public:
 
     virtual void switchToMode();
 
+    virtual QString name();
     virtual QString helpText();
 
 signals:
@@ -23,10 +24,12 @@ public slots:
     void groupGuessed(int right, int total);
     virtual void play();
     virtual void setSequenceText();
+    void limitLength(int newValue);
 
 private:
     HighScores  m_scores;
     int         m_WPM;
+    QSpinBox   *m_groupLengthSpinBox;
 };
 
 #endif // GROUPGAMEMODE_H
